#!/usr/bin/env python3

###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, pathlib, argparse, string
import obt._globals
import obt.subspace
from obt import dep, path
from obt.deco import Deco
deco = Deco()

def print_item(key,val):
 dstr = deco.inf(subspaceid)
 kstr = deco.key(key)
 vstr = deco.val(val)
 print("%s.%s = %s"%(dstr,kstr,vstr))

###############################################################################

parser = argparse.ArgumentParser(description='obt.build subspace builder')
parser.add_argument('subspacemodulename', metavar='S', type=str, help='a subspace module to build')
parser.add_argument('--force', action="store_true", help='force rebuild' )
parser.add_argument('--wipe', action="store_true", help='wipe and rebuild' )
parser.add_argument('--buildargs', action="store", type=str, help='wipe and rebuild' )

_args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

subspacemodulename = _args["subspacemodulename"]

obt._globals.setOption("subspacemodulename",subspacemodulename)

do_wipe = False
for item in "force wipe".split(" "):
  do_wipe=True

print(subspacemodulename)

subspacemodule = obt.subspace.descriptor(subspacemodulename)

print(subspacemodule)

subspacemodule.build(_args["buildargs"],do_wipe=do_wipe)

sys.exit(0)


###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile, sys
from obt import dep, host, path, git, cmake, make, env
from obt.deco import Deco
from obt.wget import wget
import obt.command
from obt.command import Command
from obt import log

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

import _gcc

deco = Deco()

###############################################################################

class arm64_gcc(dep.StdProvider):

  def __init__(self): ############################################
    super().__init__("arm64_gcc")
    self.toolchain_dir = path.prefix()/"opt"/"toolchain"/"aarch64-elf"
    self._archlist = ["x86_64"]
    pass

  def __str__(self):
    return "Arm64-Gcc"

  ########

  def env_init(self):
    log.marker("registering Arm64-Gcc SDK")
    env.append("PATH",self.toolchain_dir/"bin")

  ########

  def build(self): ##########################################################
    gcc = _gcc.context(self)
    return gcc.build( target="aarch64-elf",
                      variant="newlib",
                      program_prefix="arm64-linux-",
                      install_prefix=self.toolchain_dir )

  ########

  def install(self): ##########################################################
    return True



#!/usr/bin/env python3

###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, pathlib, argparse, string

parser = argparse.ArgumentParser(description='obt.build dep builder')
parser.add_argument('dependency', metavar='D', type=str, help='a dependency to build')

_args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

depname = _args["dependency"]

from obt import dep, path
from obt.deco import Deco
deco = Deco()

chain = dep.Chain(depname)

def genline(dep,scope,sup,man,spres,bpres,srcr):
  line = "%-40s" % dep
  line += "%24s" % scope
  line += "%24s" % sup
  line += "%28s" % man
  line += "%28s" % spres
  line += "%28s" % bpres
  line += "%60s" % srcr
  return line

def separator():
  print(deco.inf("#########################################################################################################################################"))

separator()
print(genline(deco.white("Dependency(RevTopoOrder)"),
            deco.val("Scope"),
	          deco.val("Supported"),
	          deco.val("Manifest"),
	          deco.val("SrcPresent"),
	          deco.val("BinPresent"),
	          deco.path("SourceRoot")))
separator()
index = 0
for item in chain._list:
  should = item.should_build
  colorize_name = deco.red if should else deco.white
  name = colorize_name("%d. %s"%(index,item._name))
  sup = deco.val(item.supports_host)
  man = deco.val(item.manifest.exists())

  src_present = str(item.areRequiredSourceFilesPresent())
  if src_present=="None":
     src_present = "----"
  spres = deco.val(src_present)

  bin_present = str(item.areRequiredBinaryFilesPresent())
  if bin_present=="None":
     bin_present = "----"
  bpres = deco.val(bin_present)

  srcroot = str(item.source_root).replace(str(path.builds()),"${OBT_BUILDS}")
  srcr = deco.path(srcroot)

  scope = str(item.scope).split('.')[1]
  scope = deco.val(scope)

  print(genline(name,scope,sup,man,spres,bpres,srcr))
  index+=1

separator()



###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, pathlib, hashlib, glob
import fnmatch
from obt import buildtrace

###############################################################################

def posixpath(path):
    return '/'.join(os.path.normpath(path).split(os.sep))

###############################################################################

def patglob(path,pattern):
    path = str(path)
    l=[]
    if path[-1]!='/':
        path=path+'/'
    try:
        dirlist = os.listdir(path)
        for i in dirlist:
            ii=i
            i=path+i
            #print(i,os.path.isfile(i),pattern)
            if os.path.isfile(i):
                if fnmatch.fnmatch(os.path.basename(i),pattern):
                    #print("match %s"%i)
                    l.append(i)
    except:
        pass

    return l

###############################################################################

def recursive_patglob(path,pattern):
    path = str(path)
    l=[]
    if path[-1]!='/':
        path=path+'/'
    for i in recursive_glob_get_dirs(path):
        #print(path+i)
        l=l+recursive_patglob(path+i,pattern)
    try:
        dirlist = os.listdir(path)
        for i in dirlist:
            ii=i
            i=path+i
            #print(i,os.path.isfile(i),pattern)
            if os.path.isfile(i):
                if fnmatch.fnmatch(os.path.basename(i),pattern):
                    #print("match %s"%i)
                    l.append(i)
    except:
        pass

    return l

###############################################################################

def recursive_glob(path):
    l=[]
    if path[-1]!='/':
        path=path+'/'
    for i in recursive_glob_get_dirs(path):
        #print path+i
        l=l+recursive_glob(path+i)
    try:
        dirlist = os.listdir(path)
        for i in dirlist:
            ii=i
            i=path+i
            if os.path.isfile(i):
                #print i
                #if fnmatch.fnmatch(ii,pattern):
                #print "Matched %s" % (i)
                l.append(i)
    except:
        pass

    return l

###############################################################################

def recursive_glob_get_dirs(path):
    d=[]
    try:
        x = os.listdir(path)
        #print(x)
        for i in x:
          y = path+i
          #print(i,y)
          if os.path.isdir(y):
            d.append(os.path.basename(i))
    except:
        print("error")
        pass
    return d

############################
def md5_of_file(fname):
    fil = open(str(fname),"rb")
    data = fil.read()
    fil.close()
    md5obj = hashlib.md5()
    md5obj.update(data)
    return md5obj.hexdigest()

###############################################################################

def globber( folderbase, 
             wildcard="*", 
             subdirlist=None, 
             excludelist=[] ):
    globs = []
    filtered_globs = []
    if subdirlist==None:
        subdirlist = [ "." ]
    for s in subdirlist:
        str_glob = folderbase
        if s!=".":
             str_glob += s + '/'
        str_glob = str(str_glob)+"/"+wildcard
        print(str_glob)
        these_globs = glob.glob( str_glob )
        globs += these_globs
        #print "globbing %s" % ( str_glob )
    for s in globs:
        incfil = int(1)
        for t in excludelist:
            regex = re.compile( t )
            matchobj = re.search( t, s )
            if( matchobj ):
                #print "excluding " + s + " : (matches " + t + " )"
                incfil = int(0)
        if incfil == 1:
            filtered_globs += [ posixpath(s) ]
    #print filtered_globs
    return filtered_globs

###############################################################################
class EnumFileInfo(object):
   def __init__(self, path, size, md5):
      self.path = path
      self.size = size
      self.md5 = md5
   def __repr__(self):
        return "fileinfo[ path<%s> size<%d> md5<%s>]"%(self.path,self.size,self.md5)
###############################################################################
class EnumDirInfo(object):
    ############################################
    def __init__(self, folderbase,wildcard="*"):
        self.glob = recursive_patglob(folderbase, wildcard)
        self.contents_by_hash = dict()
        self.contents_by_path = dict()
        self.bytes_by_hash = dict()
        for item in self.glob:
            if(os.path.isfile(item)):
              h = md5_of_file(item)
              size = os.path.getsize(item)
              if h not in self.contents_by_hash:
                self.contents_by_hash[h] = []
              if h not in self.bytes_by_hash:
                self.bytes_by_hash[h] = 0
              self.bytes_by_hash[h] += size
            finfo = EnumFileInfo(item, size, h)
            self.contents_by_hash[h].append(finfo)
            self.contents_by_path[item] = finfo
    ############################################
    def numDuplicateBytes(self):
        numdupebytes = 0
        for hash in self.contents_by_hash.keys():
            file_list = self.contents_by_hash[hash]
            if len(file_list)>1:
              for item in file_list:
                numdupebytes += item.size
        return numdupebytes
    ############################################
    def numBytes(self):
        numbytes = 0
        for p in self.contents_by_path.keys():
            item = self.contents_by_path[p]
            numbytes += item.size
        return numbytes
    ############################################
    def dump(self):
        for p in self.contents_by_path:
            v = self.contents_by_path[p]
            print(v)
    ############################################
    def dump_duplicates(self):
        numdupebytes = 0
        for hash in self.contents_by_hash.keys():
            file_list = self.contents_by_hash[hash]
            if len(file_list)>1:
              #print(file_list)
              for item in file_list:
                numdupebytes += item.size
        print("numdupebytes<%d>"%numdupebytes)       
###############################################################################
def enumerateContents(folderbase, 
                      wildcard="*"):
    return EnumDirInfo(folderbase,wildcard)

###############################################################################

def mkdir(p,
          clean=False,
          parents=False):
  buildtrace.buildTrace({"op":"mkdir(%s)"%str(p),"clean":clean,"parents":parents})

  if clean:
    if p.exists():
      cmd_str = "chmod -R u+w %s"%str(p)
      print(cmd_str)
      os.system(cmd_str)
      cmd_str = "rm -rf %s"%str(p)
      print(cmd_str)
      os.system(cmd_str)
  if False==p.exists():
     p.mkdir(parents=parents)

###############################################################################
def rmdir(p,force=False):
  buildtrace.buildTrace({"op":"rmdir(%s)"%str(p)})
  if force:
    os.system("chmod -R u+wx %s"%p)
  os.system("rm -rf %s"%p)

###############################################################################

def ensureDirectoryExists(p):
  buildtrace.buildTrace({"op":"ensureDirExists(%s)"%str(p)})
  if False==p.exists():
     p.mkdir()

###############################################################################

def sizeOfDirectory(start_path):
  total_size = 0
  for dirpath, dirnames, filenames in os.walk(start_path):
    for f in filenames:
      fp = os.path.join(dirpath, f)
      total_size += os.path.getsize(fp)
  return total_size

###############################################################################

def chdir(p):
  buildtrace.buildTrace({"op":"chdir(%s)"%str(p)})
  os.chdir(str(p))

###############################################################################

def copyfile(file_from,file_dest,modeset=""):
  buildtrace.buildTrace({"op":"copyfile", "from": str(file_from), "dest": str(file_dest), "modeset": modeset})
  os.system("cp \"%s\" \"%s\"" % (str(file_from), str(file_dest)))
  if modeset!="":
    os.system("chmod %s %s"%(modeset,str(file_dest)))

def copydir(src_dir,dest_dir):
  rmdir(dest_dir)
  buildtrace.buildTrace({"op":"copydir", "from": str(src_dir), "dest": str(dest_dir)})
  os.system("cp -r %s %s"%(str(src_dir),str(dest_dir)))

def copyfiles(file_from,dest_dir,modeset=""):
  buildtrace.buildTrace({"op":"copyfiles", "from": str(file_from), "dest_dir": str(dest_dir), "modeset": modeset})
  os.system("cp -r \"%s\" \"%s/\"" % (str(file_from), str(dest_dir)))
  if modeset!="":
    os.system("chmod %s %s/*"%(modeset,str(dest_dir)))





###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, path

###############################################################################

class osgeoliblas(dep.StdProvider):
  name = "osgeoliblas"
  def __init__(self):
    super().__init__(osgeoliblas.name)
    self.declareDep("cmake")
    self.declareDep("osgeolaszip")
    self.declareDep("osgeoproj")
    self.declareDep("osgeotiff")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder.setCmVars({
      "WITH_LASZIP" : "TRUE",
      "OSGEO4W_ROOT_DIR": path.stage()
    })
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=osgeoliblas.name,
                             repospec="libLAS/libLAS",
                             revision="e6a1aaed412d638687b8aec44f7b12df7ca2bbbb",
                             recursive=False)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"liblas_c.so").exists()



#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import sys, argparse, multiprocessing
from obt import exeinfo, deco, path
deco = deco.Deco()

###########################################

parser = argparse.ArgumentParser(description='obt.exedump')
parser.add_argument('--exe', metavar="exe", help='executable to dump' )

args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

stage = str(path.stage())
project = str(path.project_root())

bin_none = []
bin_prj = []
bin_stage = []
bin_sys = []

if "exe" in args and args["exe"] != None:
  libs = exeinfo.getLinkedLibraries(args["exe"])
  for key in libs.keys():

    val = libs[key]
    val_color = deco.white

    outstr = "%-50s"%deco.cyan(key)
    outstr += " : "

    if (val!=None) and (stage in val):
       val = val.replace(stage,"<STAGE>")
       outstr += "%-60s"% deco.yellow(val)
       bin_stage += [outstr]
    elif (val!=None) and (project in val):
       val = val.replace(project,"<PRJ>")
       outstr += "%-60s"% deco.orange(val)
       bin_prj += [outstr]
    elif val==None:
       outstr += "%-60s"% deco.magenta(val)
       bin_none += [outstr]
    else:
       outstr += "%-60s"% deco.white(val)
       bin_sys += [outstr]
       
for item in bin_none:
   print(item)
for item in bin_sys:
   print(item)
for item in bin_stage:
   print(item)
for item in bin_prj:
   print(item)






#!/usr/bin/env python3
import os, sys, subprocess, argparse, logging
from yarl import URL
from pathlib import Path

parser = argparse.ArgumentParser(description='jinja-build')
parser.add_argument('--sha', metavar="sha", help='sha',required=True )
parser.add_argument('--outputdir', metavar="outputdir", help='outputdirectory',required=True )
parser.add_argument('--gitURL', metavar="gitURL", required=True )
parser.add_argument("--skipLFS",action="store_true")

args = vars(parser.parse_args())

SCRIPTDIR=Path(os.path.dirname(os.path.realpath(__file__)))

################################################################################

OUTPUTDIR = Path(args["outputdir"])
SKIPLFS = args["skipLFS"]
GITURL = args["gitURL"]
REPOBASEDIR = OUTPUTDIR/"repo"

#########################################

SHA=args["sha"]
BASEDIR=OUTPUTDIR/"builds"/SHA
REPODIR=OUTPUTDIR/"repo"

print("SCRIPTDIR=%s"%SCRIPTDIR)
print("BASEDIR=%s"%BASEDIR)
print("SHA=%s"%SHA)
print("REPODIR=%s"%REPODIR)

#########################################
os.system("mkdir -p %s"%BASEDIR)
os.system("rm -rf %s"%REPODIR)
print("Cloning %s"%GITURL)
if SKIPLFS:
  os.environ["GIT_LFS_SKIP_SMUDGE"]="1"
os.system("git clone --recursive %s %s"%(GITURL,REPODIR))
os.chdir(REPODIR)
os.system("git fetch")
os.system("git checkout %s" % SHA)
os.system("git submodule update")



###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import platform, os, pathlib,sys
import multiprocessing
import importlib.util
import obt.path 
import obt.host 

###############################################################################

def instance(identifier,module_path):
  the_module = None
  spec = importlib.util.spec_from_file_location(identifier, str(module_path))
  module_inst = importlib.util.module_from_spec(spec)
  sys.modules[identifier] = module_inst
  a = spec.loader.exec_module(module_inst)
  return module_inst

###############################################################################

def enumerate_simple(enuminterface):
  #######################################
  class EnumItem:
    def __init__(self,name,fullpath,module):
      self._name = name 
      self._fullpath = fullpath
      self._module = module
  #######################################
  hostident = obt.host.description().identifier
  #######################################
  module_dict = dict()
  module_dirs_list = os.environ["OBT_MODULES_PATH"].split(":")
  subdir = enuminterface.subdir
  for module_dir in module_dirs_list:
    module_dir_2 = obt.path.Path(module_dir)/subdir
    if module_dir_2.exists():
      #print(module_dir_2)
      path_list = os.listdir(module_dir_2)
      #print(path_list)
      for item in path_list:
        if item != "__pycache__":
          module_test_path = module_dir_2/item
          #print(item, module_test_path)
          if module_test_path.exists():
            module = enuminterface.tryAsModule(hostident,item,module_test_path)
            if module != None:
              e = EnumItem(item,module_test_path,module)
              module_dict[item] = e
  return module_dict


###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, command, pathtools, path, host, osrelease
###############################################################################
class assimp(dep.StdProvider):
  name = "assimp"
  VERSION ="obt-v5.2.5"
  def __init__(self):
    super().__init__(assimp.name)
    self.declareDep("cmake")    
    self._builder = dep.CMakeBuilder(assimp.name)
    self._builder.setCmVar("ASSIMP_BUILD_ASSIMP_TOOLS","TRUE")
    self._builder.setCmVar("ASSIMP_BUILD_ASSIMP_VIEW","TRUE")
    if host.IsDarwin:
      self._builder.setCmVar("CMAKE_CXX_FLAGS","-Wno-deprecated-declarations")
    else:
  
      self._builder.setCmVar("CMAKE_CXX_FLAGS","-Wno-maybe-uninitialized")

      desc = osrelease.descriptor()
      use_gcc_11 = desc.version_id == "23.10"
      use_gcc_11 = use_gcc_11 or (desc.version_id == "24.04")
      if use_gcc_11:
        self._builder.setCmVar("CMAKE_CXX_COMPILER","g++-11")
        self._builder.setCmVar("CMAKE_CMAKE_C_COMPILER","gcc-11")
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=assimp.name,
                             repospec="tweakoz/assimp",
                             revision=assimp.VERSION,
                             recursive=False)

  #######################################################################
  ########
  def on_build_shell(self):
    pathtools.mkdir(self.build_dest,clean=False)
    return command.subshell( directory=self.build_dest,
                             prompt = "ASSIMP",
                             environment = dict() )
  ########
  def linkenv(self): ##########################################################
    return {
        "LIBS": ["assimp"]
    }
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libassimp.so").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, path

###############################################################################

class vhacd(dep.StdProvider):
  name = "vhacd"
  def __init__(self):
    super().__init__(vhacd.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self.build_src = self.build_src/"app"
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=vhacd.name,
                             repospec="kmammou/v-hacd",
                             revision="v4.1.0",
                             recursive=False)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"app"/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libBulletDynamics.so").exists()





###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, host, command, path

###############################################################################

class nvtt(dep.StdProvider):
  name = "nvtt"
  def __init__(self):
    super().__init__(nvtt.name)
    self.declareDep("cmake")
    self.declareDep("cuda")
    self.declareDep("openexr")
    ###########################################
    ## nvtt installs with wrong rpath install name
    ##  on mac, so we fix it up here..
    ###########################################
    class FixOsx(dep.CMakeBuilder):
      def __init__(self,name):
        super().__init__(name)
      def install(self,blddir):
        if host.IsOsx:
          dylibname = "libnvtt.dylib"
          dylibpath = self.build_dest/"src"/"nvtt"/dylibname
          nvcpath = self.build_dest/"src"/"nvtt"/dylibname
          retc = command.run(["install_name_tool","-id",
                              "@rpath/../lib/"+dylibname,
                              dylibpath])
          success = retc==0
          if success:
            success = super().install(blddir)
        else: # Linux
          success = super().install(blddir)
        return success
    ###########################################
    builder_class = dep.switch(linux=dep.CMakeBuilder,
                               macos=FixOsx)
    self._builder = builder_class(nvtt.name)
    self._builder.build_dest = self.build_dest
    ###########################################
    self._builder.requires(["openexr"])
    self._builder._cmakeenv = {
      "BUILD_SHARED_LIBS": "ON",
    }
    ############################################
    # because, cuda 10 requires it - todo - make dynamic
    ############################################
    if host.IsLinux:
        CUDA = dep.instance("cuda")
        self._builder.setCmVars({
          "CMAKE_CXX_COMPILER": CUDA.cxx_compiler,
          "CMAKE_C_COMPILER": CUDA.c_compiler })
    elif host.IsOsx:
        self._builder.setCmVars({
          "CMAKE_INSTALL_NAME_DIR": "@executable_path/../lib/",
          "CMAKE_BUILD_WITH_INSTALL_RPATH": "ON"})

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=nvtt.name,
                             repospec="tweakoz/nvidia-texture-tools",
                             revision="toz_orkdotbuild",
                             recursive=False)

  #######################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libnvimage.so").exists()


#!/usr/bin/env python3

import os, sys, pathlib, argparse
from obt import dep, host, path
import obt.pathtools
import obt._globals

assert(os.environ["OBT_SUBSPACE"]=="host")

##########################################
# build dep dict
##########################################

depnodes = obt.dep.DepNode.ALL()

##########################################

import obt.deco
deco = obt.deco.Deco()

line_index = 0
for key in sorted(depnodes):
	odd = line_index&1
	val = str(depnodes[key])
	if val!="???":
	  col_k = deco.rgbstr(255,255,0,key) if odd else deco.rgbstr(192,192,0,key)
	  col_v = deco.rgbstr(255,255,255,val) if odd else deco.rgbstr(192,192,192,val)
	  line = "%27s" % col_k
	  line += " " + deco.magenta(":") + " "
	  line += col_v
	  print(line)
	  line_index += 1



#!/usr/bin/env python3
import jinja2, git, datetime, os, subprocess, argparse, shlex, logging, sys, threading, time
from http import server
from yarl import URL
from pathlib import Path
import badge, uuid, urllib, sqlite3, hashfs, zmq
import orkbb, badge
import _masterimpl

################################################################################

SCRIPTDIR=Path(os.path.dirname(os.path.realpath(__file__)))

################################################################################
################################################################################
################################################################################
def sched_zmq_thread(sched):
  config = sched.config
  zmq_context = zmq.Context()
  zmq_socket = zmq_context.socket(zmq.REP)
  bindurl = "tcp://%s:%s"%(config.master_bind_addr,config.master_bind_port)
  print("master zmqserver binding to: %s"%bindurl)
  zmq_socket.bind(bindurl)
  poller = zmq.Poller()
  poller.register(zmq_socket, zmq.POLLIN)
  while True:
    if poller.poll(10*1000): # 10s timeout in milliseconds
      message = zmq_socket.recv_json()
      cmd = message["command"]
      workername = message["worker"]
      print( "got cmd<%s> from worker<%s>"%(cmd,workername))
      ##############################################
      if cmd=="worker-connect":
        sched.onWorkerConnect(workername)
        zmq_socket.send_json({
          "connected": True
        })
      ##############################################
      elif cmd=="worker-waiting":
        reply = sched.onWorkerWaiting(workername)
        zmq_socket.send_json(reply);
      ##############################################
      elif cmd=="worker-initializing":
        reply = sched.onWorkerInitializing(workername)
        zmq_socket.send_json(reply);
      ##############################################
      elif cmd=="worker-fetching":
        reply = sched.onWorkerFetching(workername)
        zmq_socket.send_json(reply);
      ##############################################
      elif cmd=="worker-building":
        reply = sched.onWorkerBuilding(workername)
        zmq_socket.send_json(reply);
      ##############################################
      elif cmd=="worker-reporting":
        reply = sched.onWorkerReporting(workername,message)
        zmq_socket.send_json(reply);
      ##############################################
    time.sleep(0.1)
################################################################################
################################################################################
################################################################################

class Scheduler:
  #########################################
  def __init__(self,projectpath,output_dir):
    self.config = _masterimpl.Config(projectpath,output_dir)
    self.workers = self.config.workers
    self.output_dir = output_dir
    self.variants_by_worker = dict()
    self.SRC_TEMPLATEPATH = SCRIPTDIR/"assets"/"index.template"
    self.SRC_LOGOPATH = SCRIPTDIR/"assets"/"OrkidLogo.png"
    self.sitelock = threading.Lock()
    self.schedlock = threading.Lock()
    self.connected_workers = {}
    #######################################
    os.system("cp %s %s"%(self.SRC_LOGOPATH,self.output_path("OrkidLogo.png")))
    os.chdir(output_dir)
    self.update_site(init=True)
    for w in self.variants_by_worker.keys():
      worker = self.config.workers[w]
      worker.start(self)
    #######################################
    self.config.start()
    #######################################
    self.thread = threading.Thread(target=sched_zmq_thread,args=(self,))
    self.thread.start()
  #########################################
  def getWorker(self,workername):
    worker = None
    self.schedlock.acquire(blocking=True)
    if workername in self.workers:
      worker = self.workers[workername]
    self.schedlock.release()
    return worker
  #########################################
  def onWorkerConnect(self,workername):
    worker = self.getWorker(workername)
    if worker!=None:
      self.schedlock.acquire(blocking=True)
      worker.connected = True
      self.connected_workers[workername]=worker
      self.schedlock.release()
  #########################################
  def onWorkerWaiting(self,workername):
    result = {}
    worker = self.getWorker(workername)
    if worker!=None:
      self.updateWorkerRepoData(worker)
      result = worker.onWaiting()
    return result
  #########################################
  def onWorkerInitializing(self,workername):
    result = {}
    worker = self.getWorker(workername)
    if worker!=None:
      result = worker.onInitializing()
    return result
  #########################################
  def onWorkerFetching(self,workername):
    result = {}
    worker = self.getWorker(workername)
    if worker!=None:
      result = worker.onFetching()
    return result
  #########################################
  def onWorkerBuilding(self,workername):
    result = {}
    worker = self.getWorker(workername)
    if worker!=None:
      result = worker.onBuilding()
    return result
  #########################################
  def onWorkerReporting(self,workername,message):
    result = {}
    worker = self.getWorker(workername)
    if worker!=None:
      result = worker.onReporting(message)
    return result
  #########################################
  def onWorkerStatus(self,worker):
    self.update_site(init=False)
  #########################################
  def updateWorkerRepoData(self,worker):
    print("worker<%s> got status<%s>"%(worker.name,worker.status.name))
    self.update_site(init=False)
  #########################################
  def output_path(self,inpath):
    return self.output_dir/inpath
  #########################################
  def load_template(self,name):
    if name == 'index.html':
      with open(self.SRC_TEMPLATEPATH,'r') as f:
        a = f.read()
        return a
    else:
      return "WTF"
  #########################################
  def update_site(self,init=False):
    self.schedlock.acquire(blocking=True)
    variant_list = list()
    self.variants_by_worker = dict()
    for w in self.config.workers.keys():
      self.variants_by_worker[w] = list()
    for vari_key in self.config.variants.keys():
      variant = self.config.variants[vari_key]
      variant.updateRepoData()
      variant_list += [variant]
      #print(variant.name,variant.workers,variant.outputdir)
      for w in variant.workers:
        self.variants_by_worker[w] += [variant]
      if init:
        self.sitelock.acquire(blocking=True)
        os.system("rm -rf %s"%variant.outputdir)
        os.system("mkdir -p %s"%variant.outputdir)
        os.system("touch %s"%(variant.index_html))
        self.sitelock.release()
      #################################
      b = badge.Data()
      b._outpath = variant.status_svg
      b.updateStatus(variant.curbuildstatus,variant.prvbuildstatus)
      self.sitelock.acquire(blocking=True)
      b.create()
      self.sitelock.release()
    ################################################################################
    # templating engine
    ################################################################################
    env = jinja2.Environment(
        loader=jinja2.FunctionLoader(self.load_template),
        autoescape=jinja2.select_autoescape(['html', 'xml'])
    )
    template = env.get_template('index.html')
    output = template.render(MASTERNAME=self.config.master_name,VARIANT_LIST=variant_list )
    self.sitelock.acquire(blocking=True)
    with open(self.output_dir/"index.html",'w') as f:
      f.write(output)
    self.sitelock.release()
    self.schedlock.release()
  #########################################
